# -----------------------------------------------------------
# Copyright (c) 2018 Lenovo AI Platform. All Rights Reserved.
# -----------------------------------------------------------

'''
Example:
# Infer mnist data
python dcgan_inference.py --output_dir=/home/$USER/samples --checkpoint_dir=/home/zhouzd2/train/dcgan --batch_size=4
'''

import sys
import os
import tensorflow as tf
import numpy as np
import scipy.misc
parentdpath1 = os.path.dirname(os.path.abspath(__file__))
parentdpath = parentdpath1[:-15] + 'applications'
path_list = [parentdpath + '/dcgan', parentdpath]
sys.path += path_list
print(sys.path)
tf.app.flags.DEFINE_integer('batch_size', 64, 'The number of samples in each batch.')
tf.app.flags.DEFINE_string("checkpoint_dir", "/home/zhouzd2/train/dcgan166", "Directory name to save the checkpoints [checkpoint]")
tf.app.flags.DEFINE_string("output_dir", "/home/zhouzd2/tmp", "Directory name to save the generation image")
from get_data import dcgan_modle

FLAGS = tf.app.flags.FLAGS

def save_images(images, size, image_path):
  return imsave(inverse_transform(images), size, image_path)

def inverse_transform(images):
  return (images+1.)/2.

def imsave(images, size, path):
  image = np.squeeze(merge(images, size))
  return scipy.misc.imsave(path, image)

def merge(images, size):
  h, w = images.shape[1], images.shape[2]
  if (images.shape[3] in (3,4)):
    c = images.shape[3]
    img = np.zeros((h * size[0], w * size[1], c))
    for idx, image in enumerate(images):
      i = idx % size[1]
      j = idx // size[1]
      img[j * h:j * h + h, i * w:i * w + w, :] = image
    return img
  elif images.shape[3]==1:
    img = np.zeros((h * size[0], w * size[1]))
    for idx, image in enumerate(images):
      i = idx % size[1]
      j = idx // size[1]
      img[j * h:j * h + h, i * w:i * w + w] = image[:,:,0]
    return img
  else:
    raise ValueError('in merge(images,size) images parameter '
                     'must have dimensions: HxW or HxWx3 or HxWx4')

def image_manifold_size(num_images):
  manifold_h = int(np.floor(np.sqrt(num_images)))
  manifold_w = int(np.ceil(np.sqrt(num_images)))
  assert manifold_h * manifold_w == num_images
  return manifold_h, manifold_w

def model_fn(data_direct, data_batch, batch_size):
    #dcgan = DCGAN()
    dcgan = dcgan_modle(FLAGS.checkpoint_dir)
    loss_dict = dcgan.build_model()
    end_point = {}
    if dcgan.dataset_name == 'mnist':
        feed_ops = [dcgan.inputs, dcgan.z, dcgan.y, dcgan.sampler]
    else:
        feed_ops = [dcgan.inputs, dcgan.z, dcgan.sampler]
    return loss_dict, end_point, feed_ops

def sess_fn(sess, feed_ops, step):
    sampler = feed_ops.pop(-1)
    z = feed_ops[1]
    feed_ops.append(sampler)
    batch_z = np.random.uniform(-1, 1, [FLAGS.batch_size, 100]) \
        .astype(np.float32)
    feed_data = {z: batch_z}
    samples = sess.run(sampler, feed_dict=feed_data)
    try:
        save_images(samples, image_manifold_size(samples.shape[0]),
                    '{}/predict_{:04d}.png'.format(FLAGS.output_dir, step))
    except:
        print("ERROR Image generation failed !...")

def main():
    ckpt_dir = FLAGS.checkpoint_dir
    graph = tf.Graph().as_default()
    with graph:
        config = tf.ConfigProto()
        config.gpu_options.allow_growth = True
        with tf.Session(config=config) as sess:
            data_direct = []
            loss_dict, end_point, feed_ops = model_fn(data_direct, None, FLAGS.batch_size)
            saver = tf.train.Saver()
            if os.path.exists(ckpt_dir):
                print("dcgan Restoring Variables from Checkpoint.")
                saver.restore(sess, tf.train.latest_checkpoint(ckpt_dir))
                #saver.restore(sess, ckpt_dir + '/model.ckpt-110000')
            else:
                print("ERROR dcgan Not able to find Checkpoint. Going to stop now...")
                raise
            for i in xrange(10):
                sess_fn(sess, feed_ops, i)
            print ("inference done ..........")

if __name__ == '__main__':
    main()