# -----------------------------------------------------------
# Copyright (c) 2018 Lenovo AI Platform. All Rights Reserved.
# -----------------------------------------------------------


"""
Example:
python fmcnn_inference.py --input_dir=/nfsdata/xianghui_data/datasets/coco/test/ --checkpoint_dir=/home/xh/train/fmrcnn --output_dir=/home/xh/tmp
"""

import tensorflow as tf
import sys, os
import numpy as np
import skimage.io
import skimage.color
from PIL import Image

parentdpath1 = os.path.dirname(os.path.abspath(__file__))
parentdpath = parentdpath1[:-16] + 'applications'
path_list = [parentdpath + '/fmrcnn', parentdpath, parentdpath1[:-16] + '/inference']
sys.path += path_list
print(sys.path)

import coco
import inference_util
from mrcnn import model
from pycocotools.coco import COCO
from libs.nets import nets_factory as network
from libs.nets import pyramid_network as pyramid_network
from libs.visualization.pil_utils import cat_id_to_cls_name, draw_img, draw_bbox,draw_bbox_new
from libs.layers import mask_decoder

tf.app.flags.DEFINE_string('prob', '0.5',
                           """ mask and segmentation probability. """)
tf.app.flags.DEFINE_string('output_dir', '/home/shangyw2/tmp',
                           """output result dir """)
tf.app.flags.DEFINE_string('category_file', 'instances_train.json',
                           """category file, default is placed in checkpoint_dir.""")
tf.app.flags.DEFINE_string('input_dir', '/nfsdata/wangpeng_data/data/fmrcnndata/coco',
                           """inference_data """)
tf.app.flags.DEFINE_string('checkpoint_dir', '/home/shangyw2/train/fmrcnn',
                           """Directory where to read training checkpoints """)
tf.app.flags.DEFINE_float(
    'weight_decay', 1e-4, 'The weight decay on the model weights.')
FLAGS = tf.app.flags.FLAGS


def main():

    with tf.Graph().as_default():

        # feed ops
        img = tf.placeholder(tf.float32, [None, None, None, 3])
        ih = tf.placeholder(tf.int32, None)
        iw = tf.placeholder(tf.int32, None)
        #boxes = tf.placeholder(tf.float32, [None, 5])
        #masks = tf.placeholder(tf.int32, [None, None, None])
        im_shape = tf.shape(img)
        image = tf.reshape(img, (im_shape[0], im_shape[1], im_shape[2], 3))

        #data prepare
        ckpt_dir = FLAGS.checkpoint_dir
        batch_size = 1

        # get_num_classes_and_labels
        coco_data = COCO("{}/{}".format(ckpt_dir, FLAGS.category_file))
        num_classes = len(sorted(coco_data.getCatIds())) + 1


        ## network FMCNN
        logits, end_points, pyramid_map = network.get_network(FLAGS.network, image,
                                                              weight_decay=FLAGS.weight_decay, is_training=True)
        outputs = pyramid_network.build(end_points, im_shape[1], im_shape[2], pyramid_map,
                                        num_classes=num_classes,
                                        base_anchors=9,
                                        is_training=False,
                                        gt_boxes=tf.convert_to_tensor([], dtype=tf.float32),
                                        gt_masks=tf.convert_to_tensor([], dtype=tf.float32),
                                        loss_weights=[0.2, 0.2, 1.0, 0.2, 1.0]
                                        )

        #total_loss = outputs['total_loss']

        #inference is begining
        config = tf.ConfigProto()
        config.gpu_options.allow_growth = True
        with tf.Session(config=config) as sess:
            saver = tf.train.Saver()
            if os.path.exists(ckpt_dir):
                saver.restore(sess, tf.train.latest_checkpoint(ckpt_dir))
                #saver.restore(sess, ckpt_dir + '/coco_resnet50_model.ckpt-270000')
            else:
                raise ValueError("Checkpoint dir must be provided to restore model.")

            for image_path in inference_util.get_img_files(FLAGS.input_dir):

                # Load image
                _image = skimage.io.imread(image_path)
                # If grayscale. Convert to RGB for consistency.
                if _image.ndim != 3:
                    _image = skimage.color.gray2rgb(_image)
                # If has an alpha channel, remove it for consistency
                if _image.shape[-1] == 4:
                    _image = _image[..., :3]
                images = np.zeros((batch_size,) + _image.shape, dtype=np.float32)
                config = coco.CocoConfig()
                images[0] = _image.astype(np.float32) - config.MEAN_PIXEL

                _base_name = os.path.basename(image_path)
                _split_name = os.path.splitext(_base_name)
                if _split_name and len(_split_name) == 2:
                  name = _split_name[0] + '_predict' + _split_name[1]

                mask_dict = outputs['mask']
                box_dict =  outputs['final_boxes']

                # inference
                final_box = outputs['final_boxes']['box']
                final_cls = outputs['final_boxes']['cls']
                final_prob = outputs['final_boxes']['prob']
                #final_gt_cls = outputs['final_boxes']['gt_cls']
                rois = outputs['ordered_rois']
                mask_final = mask_decoder(outputs['mask']['mask'], rois, outputs['mask']['cls'], images.shape[1],
                                          images.shape[2])

                feed_data = {
                    img: images,
                    ih: images.shape[1],
                    iw: images.shape[2],
                }

                box_np, class_np, pro_np,masks_np= sess.run([final_box, final_cls, final_prob,mask_final], feed_data)

                draw_bbox_new(0,
                          image_path,
                          name=name if name else 'est',
                          bbox=box_np,
                          label=class_np,
                          prob=pro_np,
                          mask=masks_np,
                          coco_data=coco_data,
                          output_dir=FLAGS.output_dir
                          )
                # Mask_gt_fortest = outputs['Mask_gt_fortest']
                # Mask_gt = outputs['Mask_gt']
                # Mask_gt_fortest_np,Mask_gt_np = sess.run([Mask_gt_fortest,Mask_gt],feed_data)

                # rois = outputs['ordered_rois']
                # mask_final = mask_decoder(outputs['mask']['mask'], rois,outputs['mask']['cls'] , images.shape[1], images.shape[2])
                # masks_np = sess.run(mask_final,feed_data)
                # draw_mask(step,
                #           image_path,
                #           masks_np)



if __name__ == '__main__':
    main()
