# -----------------------------------------------------------
# Copyright (c) 2018 Lenovo AI Platform. All Rights Reserved.
# -----------------------------------------------------------

'''
Example:
# Infer
python frcnn_inference.py --checkpoint_dir=/home/huwh1/temp/frcnn/docker-ssh --demo_net=VGGnet_test --input_dir=${HOME}/git/smallcorgi/Faster-RCNN_TF/data/demo --output_dir=/tmp/huwh1
'''
import matplotlib
matplotlib.use('Agg')

import argparse
import numpy as np
import os, sys, cv2


parentpath = os.path.dirname(os.path.abspath(__file__))
path_list = [parentpath[:-15] + 'applications',
             parentpath[:-15] + 'applications/' + 'frcnn']
sys.path += path_list

import _init_paths
import tensorflow as tf
from lib.fast_rcnn.config import cfg
from lib.fast_rcnn.test import im_detect
from lib.fast_rcnn.nms_wrapper import nms
from lib.utils.timer import Timer
from lib.networks.factory import get_network
import matplotlib.pyplot as plt
slim = tf.contrib.slim

tf.app.flags.DEFINE_string('demo_net', 'VGGnet_test', 'Network to use [vgg16]')
tf.app.flags.DEFINE_string('checkpoint_dir', None, 'checkpoint_dir path')
tf.app.flags.DEFINE_string('category_file', '', 'category file')
tf.app.flags.DEFINE_integer('frcnn_num_classes', 21, 'number of classes')
tf.app.flags.DEFINE_string('image_file', None, 'demo image file')
tf.app.flags.DEFINE_string('input_dir', None, 'input dir for inference')
tf.app.flags.DEFINE_string('output_dir', None, 'output dir for inference')
tf.app.flags.DEFINE_string('input_fname_pattern', 'jpg.bmp.png.jpeg', 'input file pattern, None for all supported formats')
tf.app.flags.DEFINE_integer('output_dpi', 72, 'dpi of output image')
tf.app.flags.DEFINE_bool("bbox_reg", True, 'bbox regression')


FLAGS = tf.app.flags.FLAGS

# modified in _read_category_file();
CLASSES = ('__background__',
           'aeroplane', 'bicycle', 'bird', 'boat',
           'bottle', 'bus', 'car', 'cat', 'chair',
           'cow', 'diningtable', 'dog', 'horse',
           'motorbike', 'person', 'pottedplant',
           'sheep', 'sofa', 'train', 'tvmonitor')


#CLASSES = ('__background__','person','bike','motorbike','car','bus')

def _read_category_file():
    global CLASSES
    if FLAGS.category_file:
      category_file = FLAGS.category_file
    else:
      category_file = os.path.join(FLAGS.checkpoint_dir, "category.txt")
    _classes_auto = []
    with open(category_file) as f:
        lines = f.readlines()
    for line in lines:
        line = line.lower().strip('\n')
        _classes_auto.append(line)
    CLASSES = tuple(_classes_auto)
    FLAGS.frcnn_num_classes = len(CLASSES)


def vis_detections(im, class_name, dets,ax, thresh=0.5):
    """Draw detected bounding boxes."""
    inds = np.where(dets[:, -1] >= thresh)[0]
    if len(inds) == 0:
        return

    for i in inds:
        bbox = dets[i, :4]
        score = dets[i, -1]

        ax.add_patch(
            plt.Rectangle((bbox[0], bbox[1]),
                          bbox[2] - bbox[0],
                          bbox[3] - bbox[1], fill=False,
                          edgecolor='red', linewidth=3.5)
            )
        ax.text(bbox[0], bbox[1] - 2,
                '{:s} {:.3f}'.format(class_name, score),
                bbox=dict(facecolor='blue', alpha=0.5),
                fontsize=14, color='white')

    ax.set_title(('{} detections with '
                  'p({} | box) >= {:.1f}').format(class_name, class_name,
                                                  thresh),
                  fontsize=14)
    plt.axis('off')
    plt.tight_layout()
    plt.draw()


def demo(sess, net, image_name):
    """Detect object classes in an image using pre-computed object proposals."""

    # Load the demo image
    # im_file = os.path.join(cfg.DATA_DIR, 'demo', image_name)

    im_file = image_name

    #im_file = os.path.join('/home/corgi/Lab/label/pos_frame/ACCV/training/000001/',image_name)
    im = cv2.imread(im_file)

    # Detect all object classes and regress object bounds
    timer = Timer()
    timer.tic()
    scores, boxes = im_detect(sess, net, im, FLAGS.bbox_reg, bbox_reg=FLAGS.bbox_reg)
    timer.toc()
    print ('Detection took {:.3f}s for '
           '{:d} object proposals').format(timer.total_time, boxes.shape[0])

    # Visualize detections for each class
    im = im[:, :, (2, 1, 0)]
    fig, ax = plt.subplots(figsize=(12, 12))
    ax.imshow(im, aspect='equal')

    CONF_THRESH = 0.8
    NMS_THRESH = 0.3
    for cls_ind, cls in enumerate(CLASSES[1:]):
        cls_ind += 1 # because we skipped background
        cls_boxes = boxes[:, 4*cls_ind:4*(cls_ind + 1)]
        cls_scores = scores[:, cls_ind]
        dets = np.hstack((cls_boxes,
                          cls_scores[:, np.newaxis])).astype(np.float32)
        keep = nms(dets, NMS_THRESH, 'gpu:0')
        dets = dets[keep, :]
        vis_detections(im, cls, dets, ax, thresh=CONF_THRESH)

    return fig

# def parse_args():
#     """Parse input arguments."""
#     parser = argparse.ArgumentParser(description='Faster R-CNN demo')
#     parser.add_argument('--gpu', dest='gpu_id', help='GPU device id to use [0]',
#                         default=0, type=int)
#     parser.add_argument('--cpu', dest='cpu_mode',
#                         help='Use CPU mode (overrides --gpu)',
#                         action='store_true')
#     parser.add_argument('--net', dest='demo_net', help='Network to use [vgg16]',
#                         default='VGGnet_test')
#     parser.add_argument('--model', dest='model', help='Model path',
#                         default=' ')
#
#     args = parser.parse_args()
#
#     return args
if __name__ == '__main__':
    cfg.TEST.HAS_RPN = True  # Use RPN for proposals

    # args = parse_args()
    args = FLAGS

    if args.checkpoint_dir == ' ':
        raise IOError(('Error: checkpoint_dir not found.\n'))

    # read CLASSES
    _read_category_file()

    # init session
    sess = tf.Session(config=tf.ConfigProto(allow_soft_placement=True))
    # load network
    net = get_network(args.demo_net)
    # load model
    # saver = tf.train.Saver(write_version=tf.train.SaverDef.V1)
    # saver.restore(sess, args.model)
    variables_to_restore = slim.get_variables_to_restore()
    saver = tf.train.Saver(variables_to_restore)
    ckpt = tf.train.get_checkpoint_state(args.checkpoint_dir)
    saver.restore(sess, ckpt.model_checkpoint_path)

    #sess.run(tf.initialize_all_variables())

    print '\n\nLoaded network {:s}'.format(args.checkpoint_dir)

    # Warmup on a dummy image
    im = 128 * np.ones((300, 300, 3), dtype=np.uint8)
    for i in xrange(2):
        _, _= im_detect(sess, net, im, bbox_reg=FLAGS.bbox_reg)

    # im_names = ['000456.jpg', '000542.jpg', '001150.jpg',
    #             '001763.jpg', '004545.jpg']


    # for im_name in im_names:
    #     print '~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~~'
    #     print 'Demo for data/demo/{}'.format(im_name)
    #     demo(sess, net, im_name)

    if FLAGS.image_file:
        image_file = FLAGS.image_file
        print("Demo for {}".format(image_file))
        demo(sess, net, image_file)

        plt.show()

    if FLAGS.input_dir and FLAGS.output_dir:
        print("apply inference in {}, output to {}".format(FLAGS.input_dir, FLAGS.output_dir))
        # get input file pattern
        if FLAGS.input_fname_pattern:
            pattern = str(FLAGS.input_fname_pattern).split(".")
        else:
            pattern = None
            print("inference on all files")
        # get output dir
        if not tf.gfile.Exists(FLAGS.output_dir):
            tf.gfile.MakeDirs(FLAGS.output_dir)

        # get input images list
        for dirpath, dirnames, filenames in os.walk(FLAGS.input_dir):
            for filename in filenames:
                if pattern:
                    p = [x.lower() for x in pattern]
                else:
                    p = filename.split(".")[-1]
                _split_name = os.path.splitext(filename)
                if _split_name[1][1:].lower() in p:
                    # get output image
                    output_image = _split_name[0] + "_predict." + _split_name[1][1:]
                    output_image = os.path.join(FLAGS.output_dir, output_image)
                    input_image = os.path.join(dirpath, filename)
                    print('Demo for {}'.format(input_image))
                    predict_fig = demo(sess, net, input_image)

                    # save figure
                    predict_fig.savefig(fname=output_image, dpi=FLAGS.output_dpi, format=_split_name[1][1:])
                    print("Saved to {}".format(output_image))
