# -----------------------------------------------------------
# Copyright (c) 2018 Lenovo AI Platform. All Rights Reserved.
# -----------------------------------------------------------


"""
Example:
python maskrcnn_inference.py --input_dir=/home/xh/data/pics --checkpoint_dir=/home/xh/Mask_RCNN --output_dir=/home/xh/tmp/maskrcnn
"""

import warnings
warnings.filterwarnings("ignore", message="numpy.dtype size changed")
warnings.filterwarnings("ignore", message="numpy.ufunc size changed")

import os
import sys
import random
import math
import numpy as np
import matplotlib

import tensorflow as tf
import sys, os
import numpy as np


parentdpath1 = os.path.dirname(os.path.abspath(__file__))
parentdpath = parentdpath1[:-19] + '/applications'
path_list = [parentdpath,
             parentdpath + '/maskrcnn',
             parentdpath + '/maskrcnn/samples',
             parentdpath + '/maskrcnn/samples/coco',
             parentdpath1[:-19] + '/inference',
             parentdpath1[:-19] + '/platform',]
sys.path += path_list
print(sys.path)

tf.app.flags.DEFINE_string('output_dir', '', """output result dir """)
tf.app.flags.DEFINE_string('input_dir', '', """inference_data """)
tf.app.flags.DEFINE_string('weights_file', 'mask_rcnn_coco.h5', """pretrained_weights_file """)
tf.app.flags.DEFINE_string('checkpoint_dir', '', """Directory where to load pretrained weights """)
tf.app.flags.DEFINE_string('category_file', 'instances_train.json', """category file, default is placed in checkpoint_dir.""")
FLAGS = tf.app.flags.FLAGS

# Root directory of the project
ROOT_DIR = os.path.abspath("/tmp")

import inference_util
# Import Mask RCNN
from pycocotools.coco import COCO
from mrcnn import utils, visualize
import mrcnn.model as modellib
# Import COCO config
from letrain_keras.config import Config

import matplotlib.pyplot as plt
import skimage.io
import skimage.color
from PIL import Image
# Directory to save logs and trained model
MODEL_DIR = os.path.join(ROOT_DIR, "logs")
def find_last(model_dir):
    """Finds the last checkpoint file of the last trained model in the
    model directory.
    Returns:
        The path of the last checkpoint file
    """
    # Get directory names. Each directory corresponds to a model
    dir_name = os.path.join(model_dir, 'task_0')
    # Find the last checkpoint
    checkpoints = next(os.walk(dir_name))[2]
    checkpoints = filter(lambda f: f.startswith("mask_rcnn"), checkpoints)
    checkpoints = sorted(checkpoints)
    if not checkpoints:
        import errno
        raise IOError(
            errno.ENOENT, "Could not find weight files in {}".format(dir_name))
    checkpoint = os.path.join(dir_name, checkpoints[-1])
    return checkpoint
# Local path to trained weights file
#COCO_MODEL_PATH = os.path.join(FLAGS.checkpoint_dir, FLAGS.weights_file)
# Download COCO trained weights from Releases if needed
#if not os.path.exists(COCO_MODEL_PATH):
#   raise ValueError("No %s found." % COCO_MODEL_PATH)

class CocoConfig(Config):
    """Configuration for training on MS COCO.
    Derives from the base Config class and overrides values specific
    to the COCO dataset.
    """
    # Give the configuration a recognizable name
    NAME = "coco"

    # We use a GPU with 12GB memory, which can fit two images.
    # Adjust down if you use a smaller GPU.
    IMAGES_PER_GPU = 1

    # Uncomment to train on 8 GPUs (default is 1)
    # GPU_COUNT = 8

    def __init__(self, num_classes, GPU_COUNT=1):
        """Set values of computed attributes."""
        # Number of classes (including background)
        # NUM_CLASSES = 1 + 80  # COCO has 80 classes
        self.NUM_CLASSES = num_classes
        super(CocoConfig, self).__init__()

class InferenceConfig(CocoConfig):
    # Set batch size to 1 since we'll be running inference on
    # one image at a time. Batch size = GPU_COUNT * IMAGES_PER_GPU
    GPU_COUNT = 1
    IMAGES_PER_GPU = 1


# COCO Class names
# Index of the class in the list is its ID. For example, to get ID of
# the teddy bear class, use: class_names.index('teddy bear')

# get labels from coco datasets
coco_annotation_file = os.path.join(FLAGS.checkpoint_dir, FLAGS.category_file)
if os.path.exists(coco_annotation_file) and os.path.isfile(coco_annotation_file):
    class_names = ['background']
    coco_data = COCO("{}/{}".format(FLAGS.checkpoint_dir, FLAGS.category_file))
    categories = coco_data.dataset['categories']
    for cg in categories:
        class_names.append(cg['name'])
else:
    class_names = ['BG', 'person', 'bicycle', 'car', 'motorcycle', 'airplane',
                   'bus', 'train', 'truck', 'boat', 'traffic light',
                   'fire hydrant', 'stop sign', 'parking meter', 'bench', 'bird',
                   'cat', 'dog', 'horse', 'sheep', 'cow', 'elephant', 'bear',
                   'zebra', 'giraffe', 'backpack', 'umbrella', 'handbag', 'tie',
                   'suitcase', 'frisbee', 'skis', 'snowboard', 'sports ball',
                   'kite', 'baseball bat', 'baseball glove', 'skateboard',
                   'surfboard', 'tennis racket', 'bottle', 'wine glass', 'cup',
                   'fork', 'knife', 'spoon', 'bowl', 'banana', 'apple',
                   'sandwich', 'orange', 'broccoli', 'carrot', 'hot dog', 'pizza',
                   'donut', 'cake', 'chair', 'couch', 'potted plant', 'bed',
                   'dining table', 'toilet', 'tv', 'laptop', 'mouse', 'remote',
                   'keyboard', 'cell phone', 'microwave', 'oven', 'toaster',
                   'sink', 'refrigerator', 'book', 'clock', 'vase', 'scissors',
                   'teddy bear', 'hair drier', 'toothbrush']

# Load image from the images folder
#file_names = next(os.walk(FLAGS.input_dir))[2]
config = InferenceConfig(len(class_names), 1)
config.display()

# Create model object in inference mode.
model = modellib.MaskRCNN(mode="inference", model_dir=FLAGS.output_dir, config=config)

coco_model_path = find_last(FLAGS.checkpoint_dir)
# Load weights trained on MS-COCO
model.load_weights(coco_model_path, by_name=True)

file_names = inference_util.get_img_files(FLAGS.input_dir)
for file_name in file_names:
    f_name = os.path.basename(file_name)
    prefix = os.path.splitext(f_name)[0]
    postfix = os.path.splitext(f_name)[1]
    # png image has shape [x,x,4] ane expect [x,x,3]
    if postfix in [".png"]:
      im = Image.open(file_name)
      rgb_im = im.convert('RGB')
      file_name = os.path.join(FLAGS.output_dir, prefix + '.jpg')
      rgb_im.save(file_name)

    image = skimage.io.imread(file_name)
    if len(image.shape)!=3 or image.shape[2]!=3:
        msg = '{} is not a colorful image.'.format(file_name) \
            +' We will skip it.'
        print msg
        continue
    
    # Run detection
    results = model.detect([image], verbose=1)

    # Visualize results
    r = results[0]
    visualize.display_instances(image, r['rois'], r['masks'], r['class_ids'],
                                class_names, r['scores'],
                                save_dir=FLAGS.output_dir,
                                image_name=f_name)
