# -----------------------------------------------------------
# Copyright (c) 2018 Lenovo AI Platform. All Rights Reserved.
# -----------------------------------------------------------

'''
Example:
# Infer
python translate_inference.py --checkpoint_dir=/home/caozs1/train/trainslate   --input_file=/home/caozs1/tmp/input_file.txt --output_dir=/home/caozs1/tmp
'''

from __future__ import print_function
import sys
import os
import logging
import tensorflow as tf
import numpy as np
parentdpath1 = os.path.dirname(os.path.abspath(__file__))
parentdpath = parentdpath1[:-19] + 'applications'
path_list = [parentdpath + '/rnn-multi-nodes/translate', parentdpath]
sys.path += path_list
print(sys.path)
import data_utils
import seq2seq_model_orig_v1

tf.app.flags.DEFINE_integer("size", 512, "Size of each model layer.")
tf.app.flags.DEFINE_integer("num_layers", 3, "Number of layers in the model.")
tf.app.flags.DEFINE_integer("from_vocab_size", 40000, "English vocabulary size.")

tf.app.flags.DEFINE_integer("to_vocab_size", 40000, "French vocabulary size.")
# tf.app.flags.DEFINE_string('data_dir', '/nfsdata/wangpeng_data/data/translatedata/training','vocab file path')
tf.app.flags.DEFINE_integer('batch_size', 1, 'The number of samples in each batch.')
tf.app.flags.DEFINE_string(
    'checkpoint_dir', '/home/zhouzd2/train/translate/',
    'Directory where checkpoints and event logs are written to.')
tf.app.flags.DEFINE_string('input_file', '/home/zhouzd2/tmp/input_file.txt','inference input file path')
tf.app.flags.DEFINE_string('output_dir', '/home/zhouzd2/tmp','inference output file path')

FLAGS = tf.app.flags.FLAGS
_buckets = [(5, 10), (10, 15), (20, 25), (40, 50)]


def read_data(input_file):
    input_data = []
    with tf.gfile.GFile(input_file, mode="r") as input_file:
        input_line = input_file.readline()
        while input_line:
            input_data.append(input_line)
            input_line = input_file.readline()
    return input_data

def write_data(output_data, output_dir, input_file):
    file_name = input_file.split('/').pop(-1)
    output_file = output_dir + '/' + file_name + '.predict'
    try:
        with open(output_file, 'w') as f:
            for output_line in output_data:
                f.write('{}\n'.format(output_line))
        print("Translation file generation completed")
    except:
        print("ERROR generation translate file  fail !")
        raise

def create_model(session):
    dtype = tf.float32
    model = seq2seq_model_orig_v1.Seq2SeqModel(
        FLAGS.from_vocab_size,  # 40000
        FLAGS.to_vocab_size,  # 40000
        _buckets,
        FLAGS.size,  # 1024
        FLAGS.num_layers,  # 3
        FLAGS.batch_size,
        dtype=dtype)
    loss = model.get_seq2seq_loss(forward_only=True)
    saver = tf.train.Saver()
    ckpt = tf.train.get_checkpoint_state(FLAGS.checkpoint_dir)
    if ckpt and tf.train.checkpoint_exists(ckpt.model_checkpoint_path):
        print("Reading model parameters from %s" % ckpt.model_checkpoint_path)
        saver.restore(session, ckpt.model_checkpoint_path)
    else:
        print("Created model with fresh parameters.")
        session.run(tf.global_variables_initializer())
    return model


def decode():
  input_file = FLAGS.input_file
  output_dir = FLAGS.output_dir
  output_data = []
  with tf.Session() as sess:
    # Create model and load parameters.
    model = create_model(sess)
    model.batch_size = 1  # We decode one sentence at a time.

    # Load vocabularies.
    en_vocab_path = os.path.join(FLAGS.checkpoint_dir,
                                 "vocab%d.from" % FLAGS.from_vocab_size)
    fr_vocab_path = os.path.join(FLAGS.checkpoint_dir,
                                 "vocab%d.to" % FLAGS.to_vocab_size)
    en_vocab, _ = data_utils.initialize_vocabulary(en_vocab_path)
    _, rev_fr_vocab = data_utils.initialize_vocabulary(fr_vocab_path)

    input_data = read_data(input_file)
    for sentence in input_data:
      # Get token-ids for the input sentence.
      token_ids = data_utils.sentence_to_token_ids(tf.compat.as_bytes(sentence), en_vocab)
      # Which bucket does it belong to?
      bucket_id = len(_buckets) - 1
      for i, bucket in enumerate(_buckets):
        if bucket[0] >= len(token_ids):
          bucket_id = i
          break
      else:
        logging.warning("Sentence truncated: %s", sentence)

      # Get a 1-element batch to feed the sentence to the model.
      encoder_inputs, decoder_inputs, target_weights = model.get_batch(
          {bucket_id: [(token_ids, [])]}, bucket_id)
      # Get output logits for the sentence.
      _, _, output_logits = model.step(sess, encoder_inputs, decoder_inputs,
                                       target_weights, bucket_id)
      # This is a greedy decoder - outputs are just argmaxes of output_logits.
      outputs = [int(np.argmax(logit, axis=1)) for logit in output_logits]
      # If there is an EOS symbol in outputs, cut them at that point.
      if data_utils.EOS_ID in outputs:
        outputs = outputs[:outputs.index(data_utils.EOS_ID)]
      out_len = len(rev_fr_vocab)
      outputs = map(lambda x: x if x < out_len else 3, outputs)
      translate_line = " ".join([tf.compat.as_str(rev_fr_vocab[output]) for output in outputs])
      output_data.append(translate_line)
    write_data(output_data, output_dir, input_file)


if __name__ == '__main__':
    decode()
