# -----------------------------------------------------------
# Copyright (c) 2018 Lenovo AI Platform. All Rights Reserved.
# -----------------------------------------------------------

'''
Example:
# Infer
python unet_inference.py --checkpoint_dir='/home/zhouzd2/train67' --output_dir=/home/zhouzd2/tmp --input_dir=/home/zhouzd2/letrain/applications/ehealth_unet/unet_image/training --input_fname_pattern=*.png
'''

import tensorflow as tf
import sys, os
parentdpath1 = os.path.dirname(os.path.abspath(__file__))
parentdpath = parentdpath1[:-14] + 'applications'
path_list = [parentdpath + '/ehealth_unet', parentdpath]
sys.path += path_list
print sys.path
import unet
import numpy as np
from PIL import Image
import os
import glob
import skimage.io as io
import random

tf.app.flags.DEFINE_string('checkpoint_dir', '/home/zhouzd2/train67','checkpoint dir')
tf.app.flags.DEFINE_string('output_dir', '/home/zhouzd2/tmp','output result dir')
tf.app.flags.DEFINE_string('input_dir', '/nfsdata/wangpeng_data/data/unet_data/training/image','inferce data dir')
tf.app.flags.DEFINE_string("input_fname_pattern", "*.png", "Glob pattern of filename of input images [*]")

FLAGS = tf.app.flags.FLAGS

def combine_img_prediction(data, pred):
    """
    Combines the data, grouth thruth and the prediction into one rgb image
    :param data: the data tensor
    :param gt: the ground thruth tensor
    :param pred: the prediction tensor

    :returns img: the concatenated rgb image
    """
    ny = pred.shape[2]
    ch = data.shape[3]
    img = np.concatenate((to_rgb(data.reshape(-1, ny, ch)),
                          to_rgb(pred.reshape(-1, ny, 1))), axis=1)
    return img


def to_rgb(img):
    """
    Converts the given array into a RGB image. If the number of channels is not
    3 the array is tiled such that it has 3 channels. Finally, the values are
    rescaled to [0,255)

    :param img: the array to convert [nx, ny, channels]

    :returns img: the rgb image [nx, ny, 3]
    """
    img = np.atleast_3d(img)
    channels = img.shape[2]
    if channels < 3:
        img = np.tile(img, 3)
    eps = 1e-5
    img[np.isnan(img)] = 0
    img -= np.amin(img)
    img = img / (np.amax(img) + eps)
    img *= 255
    return img


def save_image(img, path):
    """
    Writes the image to disk

    :param img: the rgb image to save
    :param path: the target path
    """
    Image.fromarray(img.round().astype(np.uint8)).save(path, 'JPEG', dpi=[300, 300], quality=90)

def error_rate(predictions, labels):
    """
    Return the error rate based on dense predictions and 1-hot labels.
    """
    return 100.0 - (
        100.0 *
        np.sum(np.argmax(predictions, 3) == labels) /
        (predictions.shape[0] * predictions.shape[1] * predictions.shape[2]))

def DataGenerator(image_path,as_gray = True):
    for i in range(len(image_path)):
        img = io.imread(image_path[i],as_gray = as_gray, as_grey = as_gray)
        img = np.reshape(img, img.shape + (1,))
        yield img

def gen_data():
    data_path = FLAGS.input_dir
    image_name_arr = glob.glob(os.path.join(data_path, FLAGS.input_fname_pattern))
    image = DataGenerator(image_name_arr)
    return image, image_name_arr

def main():
    ckpt_dir = FLAGS.checkpoint_dir
    batch_size = 1
    save_image_dir = FLAGS.output_dir
    image, image_name_arr = gen_data()
    graph = tf.Graph().as_default()
    with graph:
        #load data
        config = tf.ConfigProto()
        config.gpu_options.allow_growth = True
        with tf.Session(config=config) as sess:
            Input_channels = 1
            Nb_classes = 2
            net = unet.Unet(channels=Input_channels, n_class=Nb_classes, cost_kwargs=dict(class_weights=[1, 3]),
                            layers=5,
                            features_root=32)
            saver = tf.train.Saver()
            if os.path.exists(ckpt_dir):
                print("unet Restoring Variables from Checkpoint.")
                saver.restore(sess, tf.train.latest_checkpoint(ckpt_dir))
                #saver.restore(sess, ckpt_dir + '/model.ckpt-110000')
            else:
                print("unet Not able to find Checkpoint. Going to stop now...")
                raise
            for step in xrange(len(image_name_arr)):
                batch_x = []
                for i in xrange(batch_size):
                    x = image.next()
                    batch_x.append(x)
                batch_x = np.array(batch_x, dtype=np.float32)
                prediction = sess.run(net.predicter, feed_dict={ net.x: batch_x,net.keep_prob: 1.})
                #print ("Verification error= {:.1f}%, loss= {:.4f}".format(error_rate(prediction, batch_y), loss))
                img = combine_img_prediction(batch_x, np.argmax(prediction, axis=-1))
                image_name = image_name_arr[step].split('/').pop(-1)
                name_list = image_name.split('.')
                name = name_list[0] + '_predict.' + name_list[1]
                save_image(img, "%s/%s" % (save_image_dir, name))
            print ("************ inferce end ****************")

if __name__ == '__main__':
    main()